SystemOrganization addCategory: #Trachel!

TestCase subclass: #TRTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

TRTest subclass: #TRCallbackTest
	instanceVariableNames: 'canvas circle1 circle2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

!TRCallbackTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/29/2013 10:18'!
setUp

	canvas := TRCanvas new.
	circle1 := canvas buildCircleFrom: 20 @ 20 to: 150 @ 100 color: (Color green alpha: 0.1).
	circle2 := canvas buildCircleFrom: 200 @ 200 to: 240 @ 230 color: (Color blue alpha: 0.1).	! !

!TRCallbackTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/29/2013 11:01'!
testCallback
	| callback v s  |
	callback := TRTranslationCallback 
						block: [ :shape :step | 
									v := shape. s := step ].
	circle1 addCallback: callback.

	circle1 translateTo: 20 @ 30.
	self assert: v == circle1.
	self assert: v center = (20 @ 30).
	self assert: s = (-65 @ -30).

	v := nil.
	circle1 translateBy: 5 @ 3.
	self assert: v == circle1.
	self assert: v center = (25 @ 33).
	self assert: s = (5 @ 3).
	! !

!TRCallbackTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/29/2013 10:26'!
testHasCallBack

	self deny: TREllipseShape new hasCallback.
	self assert: (TREllipseShape new addCallback: TRTranslationCallback new) hasCallback! !

TRTest subclass: #TRCameraTest
	instanceVariableNames: 'camera canvas'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

!TRCameraTest methodsFor: 'running' stamp: 'AlexandreBergel 12/29/2013 13:21'!
setUp
	super setUp. 
	canvas := TRCanvas new.
	camera := canvas camera! !

!TRCameraTest methodsFor: 'conversion' stamp: 'AlexandreBergel 12/29/2013 13:29'!
testConversion
	self assert: (camera fromSpaceToPixel: 0 @ 0) = (canvas extent / 2).
	self assert: (camera fromSpaceToPixel: 10 @ 10) = ((canvas extent / 2) + (10 @ 10)).

	self assert: (camera fromPixelToSpace: (canvas extent / 2)) = (0 @ 0).
	self assert: (camera fromPixelToSpace: ((canvas extent / 2) + (10 @ 10))) = (10 @ 10)! !

!TRCameraTest methodsFor: 'focusing' stamp: 'AlexandreBergel 12/29/2013 13:21'!
testFocusingCenter
	canvas addShape: (TREllipseShape new from: 50 @ 50 to: 100 @ 80).
	canvas camera focusOnCenter.
	self assert: canvas camera position = (75 @ 65)! !

!TRCameraTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/29/2013 11:39'!
testInitialization
	self assert: camera position = (0 @ 0)! !

!TRCameraTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/29/2013 11:40'!
testTranslation
	camera translateTo: 5 @ 3.
	self assert: camera position = (5 @ 3).
	
	camera translateBy: 2 @ 4.
	self assert: camera position = (7 @ 7).! !

TRTest subclass: #TRCanvasTest
	instanceVariableNames: 'canvas'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

!TRCanvasTest methodsFor: 'running' stamp: 'AlexandreBergel 12/14/2013 11:06'!
setUp

	canvas := TRCanvas new! !

!TRCanvasTest methodsFor: 'fixed shape' stamp: 'AlexandreBergel 12/31/2013 10:39'!
testAccessing
	| c r1 r2 |
	c := TRCanvas new.
	r1 := c buildRectangle: ((0 @ 0) extent: (150 @ 75)) color: Color red.
	r1 setAsFixed.
	
	r2 := c buildRectangle: ((0 @ 0) extent: (130 @ 50)) color: Color lightGreen.

	self assert: (c shapeForPositionInPixels: (0 @ 0)) = r1.
	self assert: (c shapeForPositionInPixels: (250 @ 250)) = r2.! !

!TRCanvasTest methodsFor: 'animations' stamp: 'AlexandreBergel 12/23/2013 17:50'!
testAddingAnimation
	canvas addAnimation: (RTLinearMove to: 50 @ 50 during: 2 on: RTBox element).
	self assert: canvas hasAnimation.
	self assert: canvas numberOfAnimations = 1! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/24/2013 15:47'!
testBox
	| rec box |
	rec := (50 @ 50 extent: 100 @ 50).
	box := canvas buildRectangle: rec color: Color purple.
	self assert: box rectangle center = rec center.

	box extent: 20 @ 20.
	self assert: box rectangle center = rec center.! !

!TRCanvasTest methodsFor: 'camera' stamp: 'AlexandreBergel 12/29/2013 11:48'!
testCamera

	self assert: canvas camera position = (0 @ 0)! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/25/2013 09:25'!
testCanvas
	| box |
	box := canvas buildRectangle: ( (5 @ 5) extent: (5 @ 5) ) color: Color red.
	self assert: box canvas == canvas.
	self assert: box athensCanvas == canvas athensCanvas.
	self assert: canvas ~~ canvas athensCanvas.! !

!TRCanvasTest methodsFor: 'events' stamp: 'AlexandreBergel 12/31/2013 08:54'!
testCanvasAndEvent

	| c f |
	c := TRCanvas new.
	f := false.
	c on: TRMouseClick do: [ :event | f := true ].
	self deny: f.
	c announce: TRMouseMove.
	self deny: f.
	c announce: TRMouseClick.
	self assert: f! !

!TRCanvasTest methodsFor: 'label positions' stamp: 'AlexandreBergel 1/1/2014 14:12'!
testCenteredLabelShapes

	| label1 label2  |
	label1 := TRLabelShape new.
	label1 text: 'hello'.
	label1 position: 10 @ 10.

	label2 := TRLabelShape new.
	label2 text: 'hello'.
	label2 leftPosition: label1 encompassingRectangle leftCenter.
	
	"
	TRCanvas new addShape: label1; addShape: label2; open
	"

	self assert: label1 encompassingRectangle = label2 encompassingRectangle! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/24/2013 18:39'!
testCircle

	| c  |
	c := TRCanvas new.
	100 to: 200 by: 20 do: [ :i |
		 c buildCircleFrom: 50 @ 50 to: i @ 100 color: (Color red alpha: 0.2).
	].
	self assert: c numberOfShapes = (100 to: 200 by: 20) size ! !

!TRCanvasTest methodsFor: 'contains' stamp: 'AlexandreBergel 12/15/2013 19:07'!
testContainsPoint1
	self assert: (canvas shapeForPosition: 20 @ 20) == canvas! !

!TRCanvasTest methodsFor: 'contains' stamp: 'AlexandreBergel 12/15/2013 19:17'!
testContainsPoint2
	| box |
	box := canvas buildRectangle: (15 @ 15 extent: 20 @ 20) color: Color gray.
	self assert: (canvas shapeForPosition: 20 @ 20) == box.
	self assert: (canvas shapeForPosition: 35 @ 35) == canvas.
	self assert: (canvas shapeForPosition: 15 @ 15) == box.
	self assert: (canvas shapeForPosition: 200 @ 200) == canvas.! !

!TRCanvasTest methodsFor: 'contains' stamp: 'AlexandreBergel 12/29/2013 10:50'!
testContainsPointAfterTranslation
	| box |
	box := canvas buildRectangle: (0 @ 0 extent: 20 @ 20) color: Color gray.

	box translateBy: 10 @ 10.
	
	self assert: (canvas shapeForPosition: 10 @ 10) == box.
	self assert: (canvas shapeForPosition: 5 @ 5) == canvas.
! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/14/2013 18:52'!
testDefault

	self assert: canvas shapes isEmpty! !

!TRCanvasTest methodsFor: 'animations' stamp: 'AlexandreBergel 12/18/2013 18:26'!
testDefaultAnimation
	self deny: canvas hasAnimation.
	self assert: canvas numberOfAnimations = 0! !

!TRCanvasTest methodsFor: 'fixed shape' stamp: 'AlexandreBergel 12/31/2013 10:16'!
testFixedShapes

	| s1 s2 |
	s1 := TRBoxShape new from: 10 @ 20 extent: 50 @ 40.
	s2 := TRBoxShape new from: 10 @ 20 extent: 50 @ 40.
	canvas addShape: s1; addShape: s2.
	
	self assert: s1 isNotFixed.
	self assert: s2 isNotFixed.
	
	s2 setAsFixed.
	self assert: s1 isNotFixed.
	self deny: s2 isNotFixed.
	
	s2 setAsNotFixed.
	self assert: s1 isNotFixed.
	self assert: s2 isNotFixed.
! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/29/2013 11:14'!
testInitialization

	| circle |
	circle := TREllipseShape new.
	self assert: circle rectangle = TRAbstractBoxShape nullRectangle! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/29/2013 22:17'!
testLabelShapes

	| label1 label2 rec |
	label1 := TRLabelShape new.
	label1 text: 'hello'.
	label2 := TRLabelShape new.
	label2 text: 'hello world'.
	self assert: label2 width > label1 width.
	
	rec := label1 encompassingRectangle.
	self assert: rec center = (0 @ 0).! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/31/2013 11:11'!
testLabelShapesAndColor

	| label  |
	label := TRLabelShape new.
	label color: Color blue.
	self assert: label color = Color blue! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/31/2013 11:06'!
testLabelShapesAndIncludesPoint

	| label rec |
	label := TRLabelShape new.
	label text: 'hello'.
	
	rec := label encompassingRectangle.
	self assert: (label includesPoint: rec topLeft).
	self assert: (label includesPoint: (rec bottomLeft - (0 @ 1))).
	self assert: (label includesPoint: rec topRight - (1 @ 0)).
	self assert: (label includesPoint: rec bottomRight - (1 @ 1)).! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/29/2013 22:05'!
testLabelShapesHeight

	| label1 label2 |
	label1 := TRLabelShape new.
	label1 text: 'hello'.
	label2 := TRLabelShape new.
	label2 text: 'hello world'.
	self assert: label2 height = label1 height! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/29/2013 10:31'!
testLine
	| line |
	line := canvas buildLineFrom: 2 @ 3 to: 13 @ 15 color: Color blue.
	self assert: line from = (2 @ 3).
	self assert: line to = (13 @ 15).
	
	line translateBy: 3 @ 4.
	self assert: line from = (5 @ 7).
	self assert: line to = (16 @ 19).
! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/14/2013 18:51'!
testShape
	| shape |
	shape := canvas buildRectangle: (5 @ 5 extent: 20 @ 20) color: Color gray.
	self assert: shape notNil.
! !

!TRCanvasTest methodsFor: 'events' stamp: 'AlexandreBergel 12/15/2013 23:17'!
testShapeAndEvent
	| shape t |
	shape := TRBoxShape new.
	t := false.
	shape on: TRMouseEnter do: [ :event | t := true ].
	self deny: t.
	shape announce: TRMouseEnter new.
	self assert: t! !

!TRCanvasTest methodsFor: 'events' stamp: 'AlexandreBergel 12/16/2013 09:49'!
testShapeAndSharedAnnouncer
	| shape1 shape2 t |
	shape1 := TRBoxShape new.
	shape2 := TRLineShape new.
	shape1 announcer: shape2 announcer.
	
	t := 0.
	shape1 on: TRMouseLeave do: [ :event | t := t + 1 ].
	shape2 on: TRMouseEnter do: [ :event | t := t + 10 ].

	shape1 announce: TRMouseEnter new.
	shape2 announce: TRMouseLeave new.
	self assert: t = 11! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/14/2013 18:52'!
testShapes

	canvas buildRectangle: (5 @ 5 extent: 20 @ 20) color: Color gray.
	canvas buildRectangle: (5 @ 5 extent: 20 @ 20) color: Color gray.
	self assert: canvas shapes size = 2! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/29/2013 10:04'!
testShapingElement

	| circle |
	circle := canvas buildCircleFrom: 20 @ 20 to: 100 @ 80 color: Color red.

	self assert: circle rectangle = ((20 @ 20) corner: (100 @ 80)).
	
	circle translateBy: -20 @ -20.
	self assert: circle rectangle = ((0 @ 0) corner: (80 @ 60)).! !

!TRCanvasTest methodsFor: 'should be drawn' stamp: 'AlexandreBergel 1/1/2014 15:56'!
testShouldBeDrawn

	self assert: (TRBoxShape new from: 0 @ 0 to: 10 @ 20) shouldBeDrawn.
	self deny: (TRBoxShape new from: 200 @ 0 to: 10 @ 0) shouldBeDrawn.
	self deny: (TRBoxShape new from: 200 @ 0 to: 200 @ 0) shouldBeDrawn.
	self deny: (TREllipseShape new from: 0 @ 0 to: 0 @ 0) shouldBeDrawn.
	self assert: (TREllipseShape new from: 10 @ 0 to: 50 @ 50) shouldBeDrawn! !

Morph subclass: #TRMorph
	instanceVariableNames: 'trachelCanvas surface eventBeginingDragging shapeBeingPointed shouldRefresh'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

!TRMorph methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 10:29'!
canvas: aTrachelCanvas
	trachelCanvas := aTrachelCanvas.
	surface := AthensCairoSurface extent: aTrachelCanvas extent! !

!TRMorph methodsFor: 'session management' stamp: 'AlexandreBergel 12/17/2013 14:56'!
createSurface
	surface := AthensCairoSurface extent: self extent asIntegerPoint! !

!TRMorph methodsFor: 'drawing' stamp: 'AlexandreBergel 12/31/2013 09:18'!
drawOn: aCanvas
	"aCanvas is a FormCanvas"
	| offset |
	
	aCanvas fillRectangle: bounds color: Color lightGray.
	trachelCanvas playAnimations.
	
	"The drawing has to be done when a change in the shapes occured or when there is an animation."
	surface drawDuring: [:cs |
		surface clear.
		
		"We display the elements that are subject to the camera"
		offset := (self extent / 2) - trachelCanvas camera position.
		cs pathTransform translateX: offset x Y: offset y.
		"cs pathTransform 
			scaleBy: 1.0."		
		trachelCanvas shapes do: [ :trachelShape |
			trachelShape drawOn: cs.
		].
	
		"We display the elements that are _NOT_ subject to the camera"
		cs pathTransform loadIdentity.
		trachelCanvas fixedShapes do: [ :trachelShape |
			trachelShape drawOn: cs.
		].
	].
	"aCanvas translucentImage: surface asForm at: self bounds origin."
	"asForm creates a new Form, which is likely to be expensive. This can be cached"
	aCanvas image: surface asForm at: self bounds origin sourceRect: (0 @ 0 extent: surface extent) rule: 34! !

!TRMorph methodsFor: 'events-processing' stamp: 'AlexandreBergel 12/15/2013 19:44'!
handleMouseMove: anEvent
	super handleMouseMove: anEvent.
	self rtMouseMoving: anEvent.! !

!TRMorph methodsFor: 'events-processing' stamp: 'AlexandreBergel 12/16/2013 15:45'!
handleMouseOver: anEvent

	| currentElement event |
	anEvent wasHandled ifTrue:[ ^ self ]. "not interested"
	"Do nothing if we are dragging"
	self isDragging ifTrue: [ ^ self ].
	 
	shapeBeingPointed ifNil: [ shapeBeingPointed := self shapeForEvent: anEvent ].
	
	currentElement := self shapeForEvent: anEvent.
	(currentElement ~~ shapeBeingPointed)
		ifTrue: [ 
			event := TRMouseLeave new.
			event morph: self.
			event position: (self relativePositionFor: anEvent).
			event shape: shapeBeingPointed.

			shapeBeingPointed announce: event.
			shapeBeingPointed := currentElement.
			self rtMouseEnter: anEvent ].
! !

!TRMorph methodsFor: 'event handling' stamp: 'AlexandreBergel 12/15/2013 19:35'!
handlesMouseDown: anEvent
	anEvent wasHandled ifTrue:[ ^ false ]. "not interested"

	^ true! !

!TRMorph methodsFor: 'morphic' stamp: 'AlexandreBergel 12/15/2013 19:34'!
handlesMouseOver: evt
	^ true! !

!TRMorph methodsFor: 'initialization' stamp: 'AlexandreBergel 12/25/2013 08:30'!
initialize
	super initialize.
	self createSurface.
	shouldRefresh := false
	"session := Smalltalk session."! !

!TRMorph methodsFor: 'testing' stamp: 'AlexandreBergel 12/15/2013 19:46'!
isDragging
	^ eventBeginingDragging notNil! !

!TRMorph methodsFor: 'morphic' stamp: 'AlexandreBergel 12/29/2013 13:37'!
layoutChanged 
	"react on morph resize"
	super layoutChanged.
	surface ifNotNil: [  
		self extent asIntegerPoint ~= surface extent ifTrue: [ 
			self createSurface.
			trachelCanvas extent: self extent ]
	].
	! !

!TRMorph methodsFor: 'event handling' stamp: 'AlexandreBergel 12/16/2013 15:57'!
mouseDown: evt

	eventBeginingDragging ifNil: [
		evt hand 
					waitForClicksOrDrag: self 
					event: evt 
					selectors: { #rtMouseClick:. nil. nil. #rtMouseDragBegin: }
					threshold: 5.
		^ self ].
	
	evt anyButtonPressed ifTrue: [
		self rtMouseDragging: evt.
	].
	evt wasHandled: true.
	^ true! !

!TRMorph methodsFor: 'event handling' stamp: 'AlexandreBergel 12/15/2013 23:08'!
mouseEnter: evt
	evt anyButtonPressed ifFalse: [
		self rtMouseEnter: evt.
	].
	evt wasHandled: true.
	^ true! !

!TRMorph methodsFor: 'event handling' stamp: 'AlexandreBergel 12/15/2013 19:34'!
mouseLeave: evt
	
	evt anyButtonPressed ifFalse: [
		self rtMouseLeave: evt.
	].
	evt wasHandled: true.
	^ true! !

!TRMorph methodsFor: 'event handling' stamp: 'AlexandreBergel 12/16/2013 15:58'!
mouseMove: evt
	
	evt anyButtonPressed
		ifTrue: [ self rtMouseDragging: evt ] 
		ifFalse: [ self rtMouseMoving: evt ].
	evt wasHandled: true.
	^ true! !

!TRMorph methodsFor: 'event handling' stamp: 'AlexandreBergel 12/16/2013 16:05'!
mouseUp: evt
	self isDragging ifTrue: 
		[self rtMouseDragEnd: evt ].
	evt wasHandled: true.! !

!TRMorph methodsFor: 'events-roassal' stamp: 'AlexandreBergel 12/31/2013 10:42'!
relativePositionFor: evt
	"Return the position within the window"
	^ evt position - self bounds origin
	"^ trachelCanvas camera fromPixelToSpace: (evt position - self bounds origin) "! !

!TRMorph methodsFor: 'event-processed' stamp: 'AlexandreBergel 12/31/2013 10:31'!
rtMouseClick: evt
	| relativePosition ev  shape |
	relativePosition := self relativePositionFor: evt.	
"	evt yellowButtonChanged
		ifTrue: [ ev := TRMouseRightClick new ].
	evt redButtonChanged
		ifTrue: [ ev := TRMouseLeftClick new ].
	evt blueButtonChanged
		ifTrue: [ ev := TRMouseClick new ]."

	shape := self shapeForPositionInPixels: relativePosition.
	ev := TRMouseClick new.
	ev commandKeyPressed: evt commandKeyPressed.
	ev controlKeyPressed: evt controlKeyPressed.
	ev shiftKeyPressed: evt shiftPressed.
	ev shape: shape.
	ev morph: self.
	
	ev position: relativePosition.
	shape announce: ev.
	! !

!TRMorph methodsFor: 'event-processed' stamp: 'AlexandreBergel 12/31/2013 10:31'!
rtMouseDragBegin: evt
	| relativePosition |
	eventBeginingDragging := evt copy.
	
	relativePosition := self relativePositionFor: evt.
	shapeBeingPointed := self shapeForPositionInPixels: relativePosition! !

!TRMorph methodsFor: 'event-processed' stamp: 'AlexandreBergel 12/16/2013 16:05'!
rtMouseDragEnd: evt
	eventBeginingDragging := nil.
	shapeBeingPointed := nil! !

!TRMorph methodsFor: 'event-processed' stamp: 'AlexandreBergel 12/31/2013 10:31'!
rtMouseDragging: evt
	| step relativePosition event  |
	eventBeginingDragging ifNil: [ ^ self ].
	relativePosition := self relativePositionFor: evt.
	step := evt position - eventBeginingDragging position.

	shapeBeingPointed ifNil: 
		[ shapeBeingPointed := self shapeForPositionInPixels: relativePosition ].
	
	event := TRMouseDragging new 
				morph: self;
				shape: shapeBeingPointed;
				step: step; 
				yourself.

"	event commandKeyPressed: evt commandKeyPressed.
	event controlKeyPressed: evt controlKeyPressed.
	event shiftKeyPressed: evt shiftPressed.				
"
	event position: relativePosition.
	shapeBeingPointed announce: event.

	eventBeginingDragging := evt copy.! !

!TRMorph methodsFor: 'event-processed' stamp: 'AlexandreBergel 12/31/2013 10:31'!
rtMouseEnter: evt
	| relativePosition ev shape |
	
	relativePosition := self relativePositionFor: evt.

	shape := self shapeForPositionInPixels: relativePosition.
	ev := TRMouseEnter new.
	ev position: relativePosition.
	ev shape: shape.
	ev morph: self.
	shape announce: ev.
! !

!TRMorph methodsFor: 'event-processed' stamp: 'AlexandreBergel 12/31/2013 10:31'!
rtMouseLeave: evt
	| position ev shape |
	
	position := self relativePositionFor: evt.
	shape := self shapeForPositionInPixels: position.
	ev := TRMouseLeave new.
	ev position: position.
	ev shape: shape.
	ev morph: self.
	shape announce: ev
! !

!TRMorph methodsFor: 'event-processed' stamp: 'AlexandreBergel 12/31/2013 10:32'!
rtMouseMoving: evt

	| relativePosition ev shape |
	relativePosition := self relativePositionFor: evt.	
	shape := self shapeForPositionInPixels: relativePosition.

	ev := TRMouseMove new.
	ev position: relativePosition.
	ev shape: shape.
	ev morph: self.
	shape announce: ev! !

!TRMorph methodsFor: 'event-processed' stamp: 'AlexandreBergel 12/31/2013 10:32'!
shapeForEvent: anEvent	
	^ self shapeForPositionInPixels: (self relativePositionFor: anEvent)! !

!TRMorph methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/31/2013 10:31'!
shapeForPositionInPixels: point
	"Return the shape located that contains point"
	 ^ trachelCanvas shapeForPositionInPixels: point! !

!TRMorph methodsFor: 'signalling' stamp: 'AlexandreBergel 12/25/2013 09:28'!
signalUpdate
	self changed! !

!TRMorph methodsFor: 'stepping' stamp: 'AlexandreBergel 12/25/2013 08:39'!
step
	self changed! !

!TRMorph methodsFor: 'stepping' stamp: 'AlexandreBergel 12/23/2013 18:59'!
stepTime
	"Answer the desired time between steps in milliseconds. This default implementation requests that the 'step' method be called once every second."

	^ 20! !

!TRMorph methodsFor: 'testing' stamp: 'AlexandreBergel 12/18/2013 23:20'!
wantsSteps
	^ trachelCanvas hasAnimation ! !

Announcement subclass: #TREvent
	instanceVariableNames: 'shape canvas position morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

!TREvent methodsFor: 'accessing' stamp: 'AlexandreBergel 12/15/2013 22:56'!
canvas
	
	^ canvas! !

!TREvent methodsFor: 'accessing' stamp: 'AlexandreBergel 12/15/2013 22:56'!
canvas: aTrachelCanvas
	
	canvas := aTrachelCanvas! !

!TREvent methodsFor: 'accessing' stamp: 'AlexandreBergel 12/18/2013 18:36'!
element
	"Could be any object"
	^ self shape element! !

!TREvent methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2013 23:44'!
morph
	^ morph! !

!TREvent methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 15:31'!
morph: aMorph
	morph := aMorph! !

!TREvent methodsFor: 'accessing' stamp: 'AlexandreBergel 12/15/2013 22:56'!
position
	
	^ position! !

!TREvent methodsFor: 'accessing' stamp: 'AlexandreBergel 12/15/2013 22:56'!
position: aPoint
	
	position := aPoint! !

!TREvent methodsFor: 'accessing' stamp: 'AlexandreBergel 12/15/2013 22:56'!
shape
	
	^ shape! !

!TREvent methodsFor: 'accessing' stamp: 'AlexandreBergel 12/15/2013 22:56'!
shape: aTrachelShape
	
	shape := aTrachelShape! !

!TREvent methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 15:32'!
signalUpdate
	morph notNil ifTrue: [ morph changed ]
	! !

TREvent subclass: #TRMouseClick
	instanceVariableNames: 'commandKeyPressed controlKeyPressed shiftKeyPressed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

!TRMouseClick commentStamp: '' prior: 0!
A ROMouseClick is xxxxxxxxx.

Instance Variables
	commandKeyPressed:		<Object>
	controlKeyPressed:		<Object>
	shiftKeyPressed:		<Object>

commandKeyPressed
	- xxxxx

controlKeyPressed
	- xxxxx

shiftKeyPressed
	- xxxxx
!

!TRMouseClick methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2013 19:08'!
commandKeyPressed
	^ commandKeyPressed! !

!TRMouseClick methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2013 19:08'!
commandKeyPressed: anObject
	commandKeyPressed := anObject! !

!TRMouseClick methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2013 19:08'!
controlKeyPressed
	^ controlKeyPressed! !

!TRMouseClick methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2013 19:08'!
controlKeyPressed: anObject
	controlKeyPressed := anObject! !

!TRMouseClick methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2013 19:08'!
initialize 
	super initialize.
	commandKeyPressed := false.
	controlKeyPressed := false.
	shiftKeyPressed := false.! !

!TRMouseClick methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2013 19:08'!
shiftKeyPressed
	^ shiftKeyPressed! !

!TRMouseClick methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2013 19:08'!
shiftKeyPressed: anObject
	shiftKeyPressed := anObject! !

TREvent subclass: #TRMouseDragging
	instanceVariableNames: 'step'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

!TRMouseDragging class methodsFor: 'public' stamp: 'AlexandreBergel 12/31/2013 09:05'!
step: aStepAsPoint
	^ self new step: aStepAsPoint! !

!TRMouseDragging methodsFor: 'initialization' stamp: 'AlexandreBergel 12/16/2013 15:51'!
initialize
	super initialize.
	step := 0 @ 0! !

!TRMouseDragging methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 15:51'!
step
	^ step! !

!TRMouseDragging methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 15:51'!
step: aPoint
	step := aPoint! !

TREvent subclass: #TRMouseEnter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

TREvent subclass: #TRMouseLeave
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

TREvent subclass: #TRMouseMove
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

Object subclass: #TRCamera
	instanceVariableNames: 'position canvas'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

!TRCamera class methodsFor: 'public' stamp: 'AlexandreBergel 12/29/2013 12:54'!
forCanvas: aTrachelCanvas

	^ super new canvas: aTrachelCanvas! !

!TRCamera class methodsFor: 'public' stamp: 'AlexandreBergel 12/29/2013 12:54'!
new
	self error: 'Instance of this class have to be created with #forCanvas:'! !

!TRCamera methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 12:56'!
canvas
	"Return the trachel canvas I am observing"
	^ canvas! !

!TRCamera methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 12:57'!
canvas: aTrachelCanvas
	"Set the canvas to observe"
	canvas := aTrachelCanvas! !

!TRCamera methodsFor: 'utility' stamp: 'AlexandreBergel 12/29/2013 13:05'!
encompassingRectangle

	"Maybe a method encompassingRectangle has to be defined in TRCanvas?"
	^ self encompassingRectangleOf: canvas shapes! !

!TRCamera methodsFor: 'utility' stamp: 'AlexandreBergel 12/29/2013 13:03'!
encompassingRectangleOf: shapes
	| encompassingRectangle |
	encompassingRectangle := shapes anyOne encompassingRectangle.
	shapes do: [ :e | encompassingRectangle := encompassingRectangle merge: e encompassingRectangle ].
	^ encompassingRectangle ! !

!TRCamera methodsFor: 'actions' stamp: 'AlexandreBergel 12/29/2013 13:07'!
focusOnCenter
	"Do nothing if no shape are present"
	canvas numberOfShapes = 0 ifTrue: [ ^ self ]. 
	self translateTo: self encompassingRectangle center! !

!TRCamera methodsFor: 'convertion' stamp: 'AlexandreBergel 12/29/2013 13:28'!
fromPixelToSpace: aPoint
	| tr |
	tr := ((canvas extent / 2) - self position) negated.
	^ AthensAffineTransform new translation: tr; transform: aPoint! !

!TRCamera methodsFor: 'convertion' stamp: 'AlexandreBergel 12/29/2013 13:22'!
fromSpaceToPixel: aPoint
	| tr |
	tr := (canvas extent / 2) - self position.
	^ AthensAffineTransform new translation: tr; transform: aPoint! !

!TRCamera methodsFor: 'initialization' stamp: 'AlexandreBergel 12/29/2013 11:37'!
initialize
	super initialize.
	position := 0 @ 0.! !

!TRCamera methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 11:37'!
position
	^ position! !

!TRCamera methodsFor: 'actions' stamp: 'AlexandreBergel 12/29/2013 11:38'!
translateBy: aPoint
	position := position + aPoint! !

!TRCamera methodsFor: 'actions' stamp: 'AlexandreBergel 12/29/2013 11:38'!
translateTo: newPosition
	position := newPosition! !

Object subclass: #TRCanvas
	instanceVariableNames: 'canvas shapes fixedShapes extent animations morph camera announcer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

!TRCanvas methodsFor: 'animations' stamp: 'AlexandreBergel 12/29/2013 20:53'!
addAnimation: anAnimation
	animations add: anAnimation.
	anAnimation addedIn: self.
	anAnimation start! !

!TRCanvas methodsFor: 'building' stamp: 'AlexandreBergel 12/31/2013 09:33'!
addFixedShape: shape
	fixedShapes add: shape.
	shape canvas: self! !

!TRCanvas methodsFor: 'menu' stamp: 'AlexandreBergel 1/1/2014 14:19'!
addMenu: aTitle callback: aBlock
	| lbl index |
	index := fixedShapes inject: 0 into: [ :s :e | s max: (e encompassingRectangle topRight x + 10) ].
	lbl := TRLabelShape new text: aTitle; leftPosition: index @ 20.
	lbl color: Color black.
	lbl on: TRMouseClick do: [ :event | aBlock value ].
	lbl on: TRMouseEnter do: [ :event | lbl color: Color blue. self signalUpdate ].
	lbl on: TRMouseLeave do: [ :event | lbl color: Color black. self signalUpdate ].
	self addFixedShape: lbl! !

!TRCanvas methodsFor: 'building' stamp: 'AlexandreBergel 12/25/2013 09:25'!
addShape: shape
	shapes add: shape.
	shape canvas: self! !

!TRCanvas methodsFor: 'events' stamp: 'AlexandreBergel 12/31/2013 08:49'!
announce: anEvent
	self announcer announce: anEvent! !

!TRCanvas methodsFor: 'events' stamp: 'AlexandreBergel 12/31/2013 08:54'!
announcer
	announcer isNil ifTrue: [ announcer := Announcer new ].
	^ announcer! !

!TRCanvas methodsFor: 'events' stamp: 'AlexandreBergel 12/31/2013 08:54'!
announcer: anAnnouncer
	announcer := anAnnouncer! !

!TRCanvas methodsFor: 'accessing' stamp: 'AlexandreBergel 12/25/2013 09:26'!
athensCanvas
	^ canvas! !

!TRCanvas methodsFor: 'building' stamp: 'AlexandreBergel 12/24/2013 18:44'!
buildCircleFrom: aPoint to: anotherPoint color: aColor
	| shape |
	shape := TREllipseShape new.
	shape from: aPoint to: anotherPoint color: aColor.
	self addShape: shape.
	^ shape! !

!TRCanvas methodsFor: 'building' stamp: 'AlexandreBergel 12/24/2013 18:35'!
buildLineFrom: aPoint to: anotherPoint color: aColor
	|  shape |
	shape := TRLineShape new.
	shape from: aPoint to: anotherPoint color: aColor.
	self addShape: shape.
	^ shape! !

!TRCanvas methodsFor: 'building' stamp: 'AlexandreBergel 12/24/2013 18:34'!
buildRectangle: aRectangle color: aColor
	| shape |
	shape := TRBoxShape new.
	shape fromRectangle: aRectangle color: aColor.
	self addShape: shape.
	^ shape! !

!TRCanvas methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 11:47'!
camera
	^ camera! !

!TRCanvas methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 13:36'!
extent
	"size of the canvas"
	^ extent! !

!TRCanvas methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 13:36'!
extent: aPoint
	"Set of the size of the canvas. Useful, for example, when the window is resized"
	extent := aPoint! !

!TRCanvas methodsFor: 'accessing' stamp: 'AlexandreBergel 12/31/2013 10:20'!
fixedShapes
	^ fixedShapes! !

!TRCanvas methodsFor: 'animations' stamp: 'AlexandreBergel 12/29/2013 13:36'!
hasAnimation
	"True is at least one animation is running"
	^ animations notEmpty! !

!TRCanvas methodsFor: 'initialization' stamp: 'AlexandreBergel 12/31/2013 09:18'!
initialize
	super initialize.
	extent := 500 @ 500.
	canvas := AthensCairoSurface extent: extent.
	shapes := OrderedCollection new.
	fixedShapes := OrderedCollection new.
	animations := OrderedCollection new.
	camera := TRCamera forCanvas: self! !

!TRCanvas methodsFor: 'testing' stamp: 'AlexandreBergel 12/31/2013 10:09'!
isFixedShape: aShape
	^ fixedShapes includes: aShape! !

!TRCanvas methodsFor: 'testing' stamp: 'AlexandreBergel 12/31/2013 10:09'!
isMovableShape: aShape
	^ shapes includes: aShape! !

!TRCanvas methodsFor: 'animations' stamp: 'AlexandreBergel 12/29/2013 13:35'!
numberOfAnimations
	"Return the number of animations contained in the canvas"
	^ animations size! !

!TRCanvas methodsFor: 'accessing' stamp: 'AlexandreBergel 12/31/2013 09:19'!
numberOfFixedShapes
	"Return the number of displayed shapes"
	^ fixedShapes size! !

!TRCanvas methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 13:35'!
numberOfShapes
	"Return the number of displayed shapes"
	^ shapes size! !

!TRCanvas methodsFor: 'events' stamp: 'AlexandreBergel 12/31/2013 08:54'!
on: event do: aBlock
	self announcer on: event do: aBlock! !

!TRCanvas methodsFor: 'instance creation' stamp: 'AlexandreBergel 12/29/2013 13:37'!
open 
	morph := TRMorph new 
		canvas: self;
		extent: canvas extent. 
	morph openInWindow! !

!TRCanvas methodsFor: 'animations' stamp: 'AlexandreBergel 12/23/2013 14:06'!
playAnimations
	| shouldClean |
	shouldClean := false.
	animations do: [ :animation | 
		animation refresh. 
		animation hasCompleted ifTrue: [ shouldClean := true ] ].
	
	shouldClean ifTrue: [ 
		animations := animations reject: [ :animation | animation hasCompleted ] ].! !

!TRCanvas methodsFor: 'actions' stamp: 'AlexandreBergel 12/31/2013 09:59'!
removeFixedShape: aShape
	fixedShapes := fixedShapes copyWithout: aShape! !

!TRCanvas methodsFor: 'actions' stamp: 'AlexandreBergel 12/31/2013 09:59'!
removeShape: aShape
	shapes := shapes copyWithout: aShape! !

!TRCanvas methodsFor: 'accessing' stamp: 'AlexandreBergel 12/31/2013 10:23'!
shapeForPosition: position
	fixedShapes do: [ :s | 
		(s includesPoint: position) ifTrue: [ ^ s ] ].

	shapes do: [ :s | 
		(s includesPoint: position) ifTrue: [ ^ s ] ].
	^ self! !

!TRCanvas methodsFor: 'accessing' stamp: 'AlexandreBergel 12/31/2013 10:43'!
shapeForPositionInPixels: position
	"position is in pixel, it corresponds to the physical location in the window.
	e.g., 0 @ 0 is top left of the window"
	fixedShapes do: [ :s | 
		(s includesPoint: position) ifTrue: [ ^ s ] ].

	shapes do: [ :s | 
		(s includesPoint: (self camera fromPixelToSpace: position)) ifTrue: [ ^ s ] ].
	^ self! !

!TRCanvas methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 10:22'!
shapes
	"Return the list of shapes"
	^ shapes! !

!TRCanvas methodsFor: 'updating' stamp: 'AlexandreBergel 12/25/2013 09:28'!
signalUpdate
	morph changed! !

Object subclass: #TRCanvasExample
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

!TRCanvasExample methodsFor: 'examples' stamp: 'AlexandreBergel 12/24/2013 18:57'!
circle
	"
	self new circle
	"
	| c  |
	c := TRCanvas new.
	c buildRectangle: (50 @ 50 corner: 80 @ 80) color: (Color blue alpha: 0.2).

	100 to: 200 by: 20 do: [ :i |
		 c buildCircleFrom: 50 @ 50 to: i @ 100 color: (Color red alpha: 0.1).
	].
	c open.
	
! !

!TRCanvasExample methodsFor: 'examples' stamp: 'AlexandreBergel 12/29/2013 11:58'!
circle2
	"
	self new circle2
	"

	| c circle line1 line2 |
	c := TRCanvas new.
	circle := TREllipseShape new from: -50 @ -50 to: 50 @ 50.
	circle color: (Color purple alpha: 0.3).
	line1 := TRLineShape new from: -50 @ 0 to: 50 @ 0.
	line2 := TRLineShape new from: 0 @ -50 to: 0 @ 50.
	line1 color: (Color blue alpha: 0.5).
	line2 color: (Color blue alpha: 0.5).
	
	c addShape: circle.
	c addShape: line1.
	c addShape: line2.
	c open! !

!TRCanvasExample methodsFor: 'examples' stamp: 'AlexandreBergel 12/31/2013 10:43'!
fixedRec
	"
	self new fixedRec
	"

	| c r1 r2 colors |
	c := TRCanvas new.
	r1 := c buildRectangle: ((0 @ 0) extent: (150 @ 75)) color: Color red.
	r2 := c buildRectangle: ((0 @ 0) extent: (130 @ 50)) color: Color lightGreen.
	c buildLineFrom: 50 @ 50 to: 80 @ 80 color: Color lightGreen.
	
	colors := Array with: Color red with: Color blue with: Color white.
	r1 on: TRMouseClick do: [ :event | event shape color: colors atRandom. c signalUpdate ].
	r2 on: TRMouseClick do: [ :event | r2 color: colors atRandom. c signalUpdate ].	
		
	r2 setAsFixed.
	
	c on: TRMouseDragging do: [ :event | c camera translateBy: event step negated. c signalUpdate ].
	c open! !

!TRCanvasExample methodsFor: 'menu' stamp: 'AlexandreBergel 1/1/2014 14:01'!
menu1
	"
	self new menu1
	"
	| canvas |
	canvas := TRCanvas new.
	canvas addMenu: '+ box' callback: [ canvas addShape: (TRBoxShape new color: (Color purple alpha: 0.1); from: (400 atRandom @ 400 atRandom) extent: (100 atRandom @ 100 atRandom)). canvas signalUpdate ].

	canvas addMenu: '+ circle' callback: [ canvas addShape: (TREllipseShape new color: (Color blue alpha: 0.1); from: (400 atRandom @ 400 atRandom) extent: (100 atRandom @ 100 atRandom)). canvas signalUpdate ].	
	canvas camera translateTo: 250 @ 250.
	canvas open! !

!TRCanvasExample methodsFor: 'examples' stamp: 'AlexandreBergel 12/14/2013 17:49'!
rec1
	"
	self new rec1
	"

	| c |
	c := TRCanvas new.
	c buildRectangle: ((50 @ 50) extent: (150 @ 75)) color: Color red.
	c buildRectangle: ((80 @ 80) extent: (150 @ 75)) color: Color lightGreen.
	c buildLineFrom: 50 @ 50 to: 80 @ 80 color: Color lightGreen.
	c open! !

!TRCanvasExample methodsFor: 'examples' stamp: 'AlexandreBergel 12/31/2013 11:20'!
text
	"
	self new text
	"
	| c label rec |
	c := TRCanvas new.
	label := TRLabelShape new.
	rec := TRBoxShape new color: Color white.
	rec fromRectangle: label encompassingRectangle.
	c addShape: rec.
	c addShape: label.
	c open! !

Object subclass: #TRFocusing
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

TRFocusing subclass: #TRFocusCenter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

!TRFocusCenter methodsFor: 'hooks' stamp: 'AlexandreBergel 12/29/2013 12:06'!
on: canvas
	| encompassingRectangle  |
	encompassingRectangle := self encompassingRectangleOf: canvas shapes.
	canvas camera translateTo: encompassingRectangle center.! !

TRFocusing subclass: #TRFocusTopLeft
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

!TRFocusTopLeft methodsFor: 'hooks' stamp: 'AlexandreBergel 12/29/2013 12:07'!
on: canvas
	| encompassingRectangle |
	encompassingRectangle := self encompassingRectangleOf: canvas shapes.
	canvas camera translateTo: encompassingRectangle topLeft! !

TRFocusing subclass: #TRFocusTopRight
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

!TRFocusTopRight methodsFor: 'hooks' stamp: 'AlexandreBergel 12/29/2013 12:07'!
on: canvas
	| encompassingRectangle |
	encompassingRectangle := self encompassingRectangleOf: canvas shapes. 
	canvas camera translateTo: encompassingRectangle topRight! !

!TRFocusing class methodsFor: 'public' stamp: 'AlexandreBergel 12/28/2013 15:14'!
on: canvas
	^ self new on: canvas! !

!TRFocusing methodsFor: 'utility' stamp: 'AlexandreBergel 12/28/2013 15:23'!
encompassingRectangleOf: shapes
	| encompassingRectangle |
	encompassingRectangle := shapes anyOne encompassingRectangle.
	shapes do: [ :e | encompassingRectangle := encompassingRectangle merge: e encompassingRectangle ].
	^ encompassingRectangle ! !

!TRFocusing methodsFor: 'hooks' stamp: 'AlexandreBergel 12/28/2013 15:14'!
on: canvas
	self subclassResponsibility! !

Object subclass: #TRShape
	instanceVariableNames: 'canvas color path strokePaint announcer element callbacks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

TRShape subclass: #TRAbstractBoxShape
	instanceVariableNames: 'rectangle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

!TRAbstractBoxShape class methodsFor: 'public' stamp: 'AlexandreBergel 12/29/2013 11:15'!
nullRectangle
	nullRectangle ifNotNil: [ ^ nullRectangle ].
	nullRectangle := (0 @ 0) extent: (0 @ 0).
	^ nullRectangle! !

!TRAbstractBoxShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 10:01'!
center
	^ rectangle center! !

!TRAbstractBoxShape methodsFor: 'hooks' stamp: 'AlexandreBergel 12/29/2013 08:15'!
computePath
	canvas ifNil: [ ^ self ].
	path := self athensCanvas
		createPath: [ :builder | 
			builder
				absolute;
				moveTo: -1 @ -1;
				lineTo: 1 @ -1;
				lineTo: 1 @ 1;
				lineTo: -1 @ 1;
				lineTo: -1 @ -1 ]! !

!TRAbstractBoxShape methodsFor: 'drawing' stamp: 'AlexandreBergel 1/1/2014 15:56'!
drawOn: athensCanvas
	self shouldBeDrawn ifFalse: [ ^ self ].
	athensCanvas pathTransform restoreAfter: [
		athensCanvas pathTransform 
			translateBy: rectangle center;
			scaleBy: rectangle extent / 2.

		athensCanvas
			setPaint: color;
			drawShape: self path.
			
"		athensCanvas
			setStrokePaint: strokePaint;
			drawShape: path
"		]! !

!TRAbstractBoxShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/28/2013 15:21'!
encompassingRectangle
	^ self rectangle! !

!TRAbstractBoxShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/24/2013 15:59'!
extent
	^ self rectangle extent! !

!TRAbstractBoxShape methodsFor: 'geometry' stamp: 'AlexandreBergel 12/24/2013 16:59'!
extent: anExtent
	| center |
	center := rectangle center.
	rectangle := (center - (anExtent / 2)) extent: anExtent.
	self resetPath! !

!TRAbstractBoxShape methodsFor: 'public' stamp: 'AlexandreBergel 12/31/2013 09:42'!
from: aPoint extent: anotherPoint 
	
	self fromRectangle: (aPoint corner: (aPoint + anotherPoint))! !

!TRAbstractBoxShape methodsFor: 'public' stamp: 'AlexandreBergel 12/16/2013 16:52'!
from: aPoint to: anotherPoint 
	
	self fromRectangle: (aPoint corner: anotherPoint)! !

!TRAbstractBoxShape methodsFor: 'public' stamp: 'AlexandreBergel 12/16/2013 16:52'!
from: aPoint to: anotherPoint color: aColor
	self from: aPoint to: anotherPoint.
	self color: aColor.! !

!TRAbstractBoxShape methodsFor: 'public' stamp: 'AlexandreBergel 12/29/2013 11:11'!
fromRectangle: r
	| oldTopLeft |
	oldTopLeft := rectangle topLeft.
	rectangle := r.
	self triggerCallbacksForStep: (r topLeft - oldTopLeft).
	self resetPath! !

!TRAbstractBoxShape methodsFor: 'public' stamp: 'AlexandreBergel 12/16/2013 16:52'!
fromRectangle: r color: aColor
	self fromRectangle: r.
	self color: aColor! !

!TRAbstractBoxShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2013 23:41'!
height
	^ rectangle height! !

!TRAbstractBoxShape methodsFor: 'testing' stamp: 'AlexandreBergel 12/31/2013 11:02'!
includesPoint: aPoint
	^ rectangle containsPoint: aPoint! !

!TRAbstractBoxShape methodsFor: 'initialization' stamp: 'AlexandreBergel 12/29/2013 11:16'!
initialize
	super initialize.
	rectangle := self class nullRectangle! !

!TRAbstractBoxShape methodsFor: 'public' stamp: 'AlexandreBergel 1/1/2014 14:29'!
leftPosition: aPosition
	self position: (aPosition - ((self width / -2) @ 0) ) asIntegerPoint! !

!TRAbstractBoxShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/24/2013 15:48'!
rectangle
	^ rectangle! !

!TRAbstractBoxShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/24/2013 18:46'!
rectangle: r
	self fromRectangle: r! !

!TRAbstractBoxShape methodsFor: 'testing' stamp: 'AlexandreBergel 1/1/2014 15:54'!
shouldBeDrawn
	^ rectangle width > 0 and: [ rectangle height > 0 ]! !

!TRAbstractBoxShape methodsFor: 'actions' stamp: 'AlexandreBergel 12/29/2013 10:55'!
translateBy: aPoint
	rectangle := (rectangle topLeft + aPoint) extent: rectangle extent.
	self triggerCallbacksForStep: aPoint! !

!TRAbstractBoxShape methodsFor: 'actions' stamp: 'AlexandreBergel 12/29/2013 10:56'!
translateTo: aPoint
	| oldTopLeft |
	oldTopLeft := rectangle topLeft.
	rectangle := (aPoint - (self extent / 2)) extent: self extent.
	self triggerCallbacksForStep: (rectangle topLeft - oldTopLeft)! !

!TRAbstractBoxShape methodsFor: 'actions' stamp: 'AlexandreBergel 12/29/2013 10:55'!
triggerCallbacksForStep: aStep
	"aStep is a point that represents a translation step"
	callbacks ifNil: [ ^ self ].
	callbacks do: [ :c | c block value: self value: aStep ]! !

!TRAbstractBoxShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2013 23:40'!
width
	^ rectangle width! !

TRAbstractBoxShape subclass: #TRBorderBoxShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

!TRBorderBoxShape methodsFor: 'drawing' stamp: 'AlexandreBergel 1/1/2014 15:57'!
drawOn: athensCanvas
	self shouldBeDrawn ifFalse: [ ^ self ].
	athensCanvas pathTransform restoreAfter: [
		athensCanvas
			setStrokePaint: strokePaint;
			drawShape: self path ]! !

TRAbstractBoxShape subclass: #TRBoxShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

TRBoxShape class
	instanceVariableNames: 'nullRectangle'!

TRBoxShape class
	instanceVariableNames: 'nullRectangle'!

!TRBoxShape methodsFor: 'drawing' stamp: 'AlexandreBergel 12/29/2013 08:13'!
drawOn: athensCanvas
	super drawOn: athensCanvas
"	athensCanvas pathTransform restoreAfter: [
		athensCanvas pathTransform 
			translateBy: position.

		athensCanvas
			setPaint: color;
			drawShape: self path ]"! !

TRAbstractBoxShape subclass: #TREllipseShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

!TREllipseShape methodsFor: 'hooks' stamp: 'AlexandreBergel 12/25/2013 09:27'!
computePath
	canvas ifNil: [ ^ self ].
	path := self athensCanvas
				createPath: [ :builder | 
					builder
						absolute;
						moveTo: 0 @ 1;
						ccwArcTo: 1.0 @ 0.0 angle: 90 degreesToRadians;
						ccwArcTo: 0.0 @ -1.0 angle: 90 degreesToRadians;
						ccwArcTo: -1.0 @ 0.0 angle: 90 degreesToRadians;
						ccwArcTo: 0 @ 1.0 angle: 90 degreesToRadians ]
"	path := canvas
		cacheAt: self
		ifAbsentPut: [ 
			canvas
				createPath: [ :builder | 
					builder
						absolute;
						moveTo: 0 @ 1;
						ccwArcTo: 1.0 @ 0.0 angle: 90 degreesToRadians;
						ccwArcTo: 0.0 @ -1.0 angle: 90 degreesToRadians;
						ccwArcTo: -1.0 @ 0.0 angle: 90 degreesToRadians;
						ccwArcTo: 0 @ 1.0 angle: 90 degreesToRadians ] ]"! !

!TREllipseShape methodsFor: 'drawing' stamp: 'AlexandreBergel 1/1/2014 15:57'!
drawOn: athensCanvas
	self shouldBeDrawn ifFalse: [ ^ self ].
	athensCanvas pathTransform restoreAfter: [
		athensCanvas pathTransform 
			translateBy: rectangle center;
			scaleBy: rectangle extent / 2.

		athensCanvas
			setPaint: color;
			drawShape: self path.
			
"		athensCanvas
			setStrokePaint: strokePaint;
			drawShape: path
"		]! !

!TREllipseShape methodsFor: 'public' stamp: 'AlexandreBergel 12/29/2013 09:59'!
from: aPoint to: anotherPoint color: aColor
	rectangle := aPoint corner: anotherPoint.
	self strokePaint: aColor.
	self color: aColor! !

!TREllipseShape methodsFor: 'testing' stamp: 'AlexandreBergel 12/24/2013 18:45'!
includesPoint: aPoint
	"We could aternatively translate rectangle by position"
	^ rectangle containsPoint: (aPoint)! !

TRShape subclass: #TRLabelShape
	instanceVariableNames: 'text position font'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

!TRLabelShape methodsFor: 'drawing' stamp: 'AlexandreBergel 1/1/2014 14:14'!
drawOn: athensCanvas
 
	athensCanvas pathTransform restoreAfter: [
		"Draw the label centered on the position"
		athensCanvas pathTransform 
			translateBy: position + ((self width / -2) @ (self height / 2)) asIntegerPoint;
			scaleBy: 2.
		athensCanvas setFont: font.
		athensCanvas setPaint: color.
		athensCanvas drawString: text
		]! !

!TRLabelShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 22:19'!
encompassingRectangle
	| w h |
	w := self width.
	h := self height.
	^ (position - ((w @ h) / 2)) extent: (w @ h)! !

!TRLabelShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 21:50'!
font
	^ font! !

!TRLabelShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 22:06'!
height
	^ font height! !

!TRLabelShape methodsFor: 'testing' stamp: 'AlexandreBergel 12/31/2013 11:03'!
includesPoint: aPoint
	^ self encompassingRectangle containsPoint: aPoint! !

!TRLabelShape methodsFor: 'initialization' stamp: 'AlexandreBergel 12/29/2013 21:50'!
initialize
	super initialize.
	position := 0 @ 0.
	text := 'Trachel label'.
	font := LogicalFont familyName: 'Arial' pointSize: 20.! !

!TRLabelShape methodsFor: 'public' stamp: 'AlexandreBergel 1/1/2014 14:17'!
leftPosition: aPosition
	self position: (aPosition - ((self width / -2) @ 0) ) asIntegerPoint! !

!TRLabelShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 22:04'!
position: aPosition
	position := aPosition! !

!TRLabelShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 22:03'!
text: aText
	text := aText! !

!TRLabelShape methodsFor: 'public' stamp: 'AlexandreBergel 12/29/2013 22:04'!
text: aText on: aPosition
	self text: aText.
	self position: aPosition! !

!TRLabelShape methodsFor: 'public' stamp: 'AlexandreBergel 1/1/2014 15:04'!
translateBy: step
	position := position + step.! !

!TRLabelShape methodsFor: 'public' stamp: 'AlexandreBergel 1/1/2014 15:04'!
translateTo: aPoint
	position := aPoint! !

!TRLabelShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 23:08'!
width
	^ (font widthOfString: text) * 2! !

TRShape subclass: #TRLineShape
	instanceVariableNames: 'from to'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

!TRLineShape methodsFor: 'hooks' stamp: 'AlexandreBergel 12/26/2013 10:35'!
computePath
	path := self athensCanvas
		createPath: [ :builder | 
			builder
				absolute;
				moveTo: from;
				lineTo: to ]! !

!TRLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/28/2013 15:22'!
encompassingRectangle
	^ Rectangle encompassing: (Array with: from with: to)! !

!TRLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 19:18'!
from
	"Return a point"
	^ from! !

!TRLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 19:18'!
from: aPoint
	from := aPoint! !

!TRLineShape methodsFor: 'public' stamp: 'AlexandreBergel 12/24/2013 16:33'!
from: aPoint to: anotherPoint
	from := aPoint.
	to := anotherPoint.
! !

!TRLineShape methodsFor: 'public' stamp: 'AlexandreBergel 12/14/2013 18:46'!
from: aPoint to: anotherPoint color: aColor

	self from: aPoint to: anotherPoint.
	self strokePaint: aColor.
	self color: aColor! !

!TRLineShape methodsFor: 'testing' stamp: 'AlexandreBergel 12/15/2013 19:16'!
includesPoint: aPoint
	^ aPoint onLineFrom: from to: to within: 3! !

!TRLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 19:18'!
to
	"Return a point"
	^ to! !

!TRLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/17/2013 16:52'!
to: aPoint
	to := aPoint! !

!TRLineShape methodsFor: 'actions' stamp: 'AlexandreBergel 12/29/2013 10:35'!
translateBy: aPoint
	from := from + aPoint.
	to := to + aPoint! !

!TRShape methodsFor: 'actions' stamp: 'AlexandreBergel 12/29/2013 10:27'!
addCallback: aCallback
	callbacks ifNil: [ callbacks := OrderedCollection new ].
	callbacks add: aCallback! !

!TRShape methodsFor: 'events' stamp: 'AlexandreBergel 12/15/2013 23:11'!
announce: anEvent
	self announcer announce: anEvent! !

!TRShape methodsFor: 'events' stamp: 'AlexandreBergel 12/15/2013 19:23'!
announcer
	announcer isNil ifTrue: [ announcer := Announcer new ].
	^ announcer! !

!TRShape methodsFor: 'events' stamp: 'AlexandreBergel 12/16/2013 09:44'!
announcer: anAnnouncer
	announcer := anAnnouncer! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/25/2013 09:26'!
athensCanvas
	^ self canvas athensCanvas! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/25/2013 09:25'!
canvas
	"Return a Trachel Canvas"
	^ canvas! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 17:47'!
canvas: aSurface
	"aCanvas is a AthensCairoSurface"
	canvas := aSurface! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 10:01'!
center
	self subclassResponsibility! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 10:10'!
color
	
	^ color! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 10:10'!
color: aColor
	
	color := aColor! !

!TRShape methodsFor: 'hooks' stamp: 'AlexandreBergel 12/24/2013 17:02'!
computePath
	self subclassResponsibility! !

!TRShape methodsFor: 'drawing' stamp: 'AlexandreBergel 12/29/2013 09:57'!
drawOn: athensCanvas

	athensCanvas pathTransform restoreAfter: [
		athensCanvas
			setPaint: color;
			drawShape: self path.
			
		athensCanvas
			setStrokePaint: strokePaint;
			drawShape: self path  ]! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 22:52'!
element
	"A shape has a reference to an object, typically the object odel that the trachel shape represents"
	^ element ! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 22:52'!
element: anObject
	"anObject could be any kind of object. A shape has a reference to an object, typically the object odel that the trachel shape represents"
	element := anObject! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/28/2013 15:21'!
encompassingRectangle
	self subclassReponsibility! !

!TRShape methodsFor: 'testing' stamp: 'AlexandreBergel 12/29/2013 10:25'!
hasCallback
	^ callbacks notNil! !

!TRShape methodsFor: 'testing' stamp: 'AlexandreBergel 12/15/2013 19:12'!
includesPoint: aPoint
	^ false! !

!TRShape methodsFor: 'initialization' stamp: 'AlexandreBergel 12/29/2013 09:57'!
initialize
	super initialize.
	color := Color gray.
	strokePaint := Color black.	
	! !

!TRShape methodsFor: 'testing' stamp: 'AlexandreBergel 12/31/2013 10:00'!
isFixed

	^ self canvas fixedShapes includes: self! !

!TRShape methodsFor: 'testing' stamp: 'AlexandreBergel 12/31/2013 10:09'!
isMovable

	^ self canvas isMovableShape: self! !

!TRShape methodsFor: 'testing' stamp: 'AlexandreBergel 12/31/2013 10:00'!
isNotFixed

	^ self isMovable! !

!TRShape methodsFor: 'events' stamp: 'AlexandreBergel 12/15/2013 23:12'!
on: event do: aBlock
	self announcer on: event do: aBlock! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/24/2013 17:02'!
path
	path ifNil: [ self computePath ].
	^ path! !

!TRShape methodsFor: 'actions' stamp: 'AlexandreBergel 12/24/2013 17:07'!
resetPath
	path := nil! !

!TRShape methodsFor: 'actions' stamp: 'AlexandreBergel 12/31/2013 10:17'!
setAsFixed
	canvas removeShape: self.
	canvas addFixedShape: self.! !

!TRShape methodsFor: 'actions' stamp: 'AlexandreBergel 12/31/2013 09:58'!
setAsNotFixed
	canvas removeFixedShape: self.
	canvas addShape: self.! !

!TRShape methodsFor: 'testing' stamp: 'AlexandreBergel 1/1/2014 15:52'!
shouldBeDrawn
	^ true! !

!TRShape methodsFor: 'actions' stamp: 'AlexandreBergel 12/29/2013 10:28'!
signalUpdate
	"Refresh the canvas"
	canvas signalUpdate! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 10:48'!
strokePaint
	"Return the color of the border line"
	^ strokePaint! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 10:49'!
strokePaint: aColor
	"set the color of the border line"
	strokePaint := aColor! !

!TRShape methodsFor: 'actions' stamp: 'AlexandreBergel 12/29/2013 10:32'!
translateBy: aPoint
	self subclassResponsibility! !

!TRShape methodsFor: 'actions' stamp: 'AlexandreBergel 12/29/2013 09:59'!
translateTo: aPoint
	self subclassResponsibility! !

Object subclass: #TRTranslationCallback
	instanceVariableNames: 'block'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel'!

!TRTranslationCallback class methodsFor: 'public' stamp: 'AlexandreBergel 12/29/2013 10:37'!
block: aBlock
	^ self new block: aBlock! !

!TRTranslationCallback methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 10:22'!
block
	^ block
	! !

!TRTranslationCallback methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 10:43'!
block: aOneArgBlock
	"aThreeArgBlock follows the pattern [ :shape | ... ]"
	block := aOneArgBlock! !

!TRTranslationCallback methodsFor: 'initialization' stamp: 'AlexandreBergel 12/29/2013 10:57'!
initialize
	super initialize.
	block := [ :shape :step | ]! !
